//-----------------------------------------------------------------------------
/// \file	Main.cpp
/// \date	12-Jan-2005
/// \author	Rob Bateman
/// \brief	The MConditionMessage is used to track certain variable flags
/// 		used for a specific feature of maya. For example, whether the
/// 		animation is playing back.
///
/// 		the `scriptJob -listConditions` command returns the following valid
/// 		types of condition:
///
///				playingBack
///				SomethingSelected
///				delete
///				autoKeyframeState
///				BatchMode
///				PolyCoreExists
///				BaseMayaExists
///				AnimationExists
///				RenderingExists
///				BaseUIExists
///				flushingScene
///				GoButtonEnabled
///				ModelExists
///				DatabaseUIExists
///				DynamicsExists
///				SubdivExists
///				KinematicsExists
///				NurbsExists
///				DeformersExists
///				PolygonsExists
///				PolyTextureExists
///				busy
///				UndoAvailable
///				RedoAvailable
///				deleteAllCondition
///				readingFile
///				alwaysWriteReferenced
///				ExplorerExists
///				ImageUIExists
///				RenderingUIExists
///				ManipsExists
///				DimensionsExists
///				ClipEditorExists
///				DevicesExists
///				AnimationUIExists
///				MayaCreatorExists
///				ModelUIExists
///				NurbsUIExists
///				SurfaceUIExists
///				PolygonsUIExists
///				DeformersUIExists
///				KinematicsUIExists
///				SubdivUIExists
///				DynamicsUIExists
///				writingFile
///				playbackIconsCondition
///				hotkeyListChange
///				OpenMayaExists
///				readingReferenceFile
///				postSceneCallbacks
///
/// Note: The inclusion of rather juicy words such as "writingFile", "busy",
/// 	  "BatchMode" etc.
//-----------------------------------------------------------------------------

// required to link the libraries
#ifdef WIN32
	#define NT_PLUGIN
	#pragma comment(lib,"Foundation.lib")
	#pragma comment(lib,"OpenMaya.lib")
	#pragma comment(lib,"OpenMayaFx.lib")
	#pragma comment(lib,"OpenMayaUi.lib")
	#pragma comment(lib,"Image.lib")
	#pragma comment(lib,"OpenMayaAnim.lib")
#endif

// under WIN32 we have to 'export' these functions so that they are visible
// inside the dll. If maya can't see the functions, it can't load the plugin!
#ifdef WIN32
	#define EXPORT __declspec(dllexport)
#else
	#define EXPORT
#endif

#include <maya/MFnPlugin.h>
#include <maya/MConditionMessage.h>
#include <maya/MGlobal.h>
#include <maya/MString.h>


//---------------------------------------------------------------------------------------------
///	\brief	This class recieves active notifications whenever the animation starts or stops.
/// 		It allows you to see if we are currently playing back animation.
///
class PlaybackState {
public:

	/// \brief	ctor	-	creates the condition callback
	///
	PlaybackState() ;

	/// dtor	-	unregisters the callback functions
	///
	~PlaybackState() ;

protected:

	/// \brief	This function is called when animation play back begins
	///
	virtual void OnStart() {
		MGlobal::displayInfo("OnStart");
	}

	/// \brief	This function is called when animation play back ends
	///
	virtual void OnStop() {
		MGlobal::displayInfo("OnStop");
	}

private:

	/// the callback ID
	MCallbackId m_Callback;

	///	The static callback function
	static void PlaybackStateChanged(bool state,void* data);
};


//---------------------------------------------------------------------------------------------
//
void PlaybackState::PlaybackStateChanged(bool state,void* data) {

	if(state)
		( (PlaybackState*)data )->OnStart();
	else
		( (PlaybackState*)data )->OnStop();
}


//---------------------------------------------------------------------------------------------
//
PlaybackState::PlaybackState() {

	m_Callback = MConditionMessage::addConditionCallback( "playingBack", PlaybackStateChanged, this );
}

//---------------------------------------------------------------------------------------------
//
PlaybackState::~PlaybackState() {
	/// make sure we delete the callback before the plugin quits !!!
	/// Maya is likely to crash otherwise!
	MConditionMessage::removeCallback( m_Callback );
}




/// pointer to the first idle object
PlaybackState* playbackMonitor;




/// \brief	This function is called once when our plugin is loaded. We can use
///			it to register any custom nodes, mel functions etc.
/// \param	obj	-	a handle to the loaded plugin
/// \return	an MStatus error code
///
EXPORT MStatus initializePlugin( MObject obj )
{
	// we need to use the plugin function set to register a new mel command with maya.
	// we can provide some info about the author of the plugin, the version number, and we
	// can also specify a required version of maya, though usually "Any" will do...
	//
	MFnPlugin fnPlugin( obj, "Rob Bateman", "1.0", "Any");

	// create the playback state monitor
	playbackMonitor = new PlaybackState();

	return MS::kSuccess;
}

/// \brief	This function is called once when our plugin is unloaded. We need
/// 		to tell maya which mel funcs, nodes etc we are removing.
/// \param	obj	-	a handle to the loaded plugin
/// \return	an MStatus error code
///
EXPORT MStatus uninitializePlugin( MObject obj)
{
	// use the plugin function set to unload the plugin
	MFnPlugin fnPlugin( obj );

	// delete the idle objects
	delete playbackMonitor;

	return MS::kSuccess;
}


